unit janColorButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TjanColorButton = class(TGraphicControl)
  private
    FGroupIndex: Integer;

    FPushDown: boolean;
    FMouseOver: boolean;
    FHotTrack: boolean;
    FColor: TColor;
    FHotColor: TColor;
    FHotFontColor: Tcolor;
    FMarginHorizontal: integer;
    FAlignment: Talignment;
    FMarginVertical: integer;
    FWordWrap: boolean;
    FBackBitmap: TBitmap;
    FDown: boolean;
    FDownColor: TColor;
    FDownFontColor: TColor;
    FFlat: boolean;
    FAllowAllUp: boolean;

    procedure UpdateExclusive;
    procedure SetGroupIndex(Value: Integer);
    procedure CMButtonPressed(var Message: TMessage); message CM_BUTTONPRESSED;

    procedure SetHotTrack(const Value: boolean);
    procedure SetColor(const Value: TColor);
    procedure SetHotColor(const Value: TColor);
    procedure SetHotFontColor(const Value: Tcolor);
    procedure AutoFit;
    procedure SetMarginHorizontal(const Value: integer);
    procedure SetAlignment(const Value: Talignment);
    procedure SetMarginVertical(const Value: integer);
    procedure SetWordWrap(const Value: boolean);
    procedure SetBackBitmap(const Value: TBitmap);
    procedure BackBitmapChanged(Sender: TObject);
    procedure SetDown(const Value: boolean);
    procedure SetDownColor(const Value: TColor);
    procedure SetDownFontColor(const Value: TColor);
    procedure SetFlat(const Value: boolean);
    procedure SetAllowAllUp(const Value: boolean);
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    { Private declarations }
  protected
    { Protected declarations }
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Click; override;
    procedure CMMouseLeave(var Message: TMessage); message CM_MouseLeave;
    procedure CMMouseEnter(var Message: TMessage); message CM_MouseEnter;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;


  public
    {With TjanColorButton you have an easy to use button where you can set the color of both text and background.}
    {}
    {In addition you can use a bitmap that will be tiled as background image.}
    {}
    {The caption can be wordwrapped if you want.}
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    {Creates and initializes an instance of TjanColorButton. }
    destructor Destroy; override;
    {Destroys an instance of TjanColorButton.}
    procedure Paint; override;
    {Renders the image of the button.}
  published
    { Published declarations }
    property GroupIndex: Integer read FGroupIndex write SetGroupIndex default 0;
    {Allows janColorButtons to work together as a group.}
    property AllowAllUp:boolean read FAllowAllUp write SetAllowAllUp;
    {Specifies whether all speed buttons in the group that contains this speed button can be unselected at the same time.}
    {}
    {For a single toggle button set GroupIndex to e.g. 1 and AllowAllUp to True}
    property Align;
    {Determines how the control aligns within its container (parent control).}
    property Alignment: Talignment read FAlignment write SetAlignment;
    {Controls the horizontal placement of the text within the button.}
    property BackBitmap: TBitmap read FBackBitmap write SetBackBitmap;
    {Determines the tiled background image of the button.}
    property Caption;
    {Specifies a text string that identifies the control to the user.}
    property WordWrap: boolean read FWordWrap write SetWordWrap;
    {Specifies whether the button text wraps when it is too long for the width of the button.}
    property Down: boolean read FDown write SetDown;
    {Specifies whether the button is selected (down) or unselected (up).}
    property Constraints;
    {Specifies the size constraints for the control.}
    {}
    {Use Constraints to specify the minimum and maximum width and height of the control. When Constraints contains maximum or minimum values, the control cant be resized to violate those constraints.}
    property Enabled;
    {Controls whether the control responds to mouse, keyboard, and timer events. }
    property Flat: boolean read FFlat write SetFlat;
    {Determines whether the button has a 3D border that provides a raised or lowered look.}
    property Font;
    {Controls the attributes of text written on the button.}
    property MarginHorizontal: integer read FMarginHorizontal write SetMarginHorizontal;
    {Determines the horizontal margin in pixels between the button caption and the borders.}
    property MarginVertical: integer read FMarginVertical write SetMarginVertical;
    {Determines the vertical margin in pixels between the button caption and the borders.}
    property Color: TColor read FColor write SetColor;
    {Determines the background color of the button.}
    property HotTrack: boolean read FHotTrack write SetHotTrack;
    {Determines whether the button will be automatically highlighted when the mouse moves over it.}
    property HotColor: TColor read FHotColor write SetHotColor;
    {Determines the HotTrack button face color.}
    property DownColor: TColor read FDownColor write SetDownColor;
    {Determines the button face color in the pressed down state.}
    property HotFontColor: TColor read FHotFontColor write SetHotFontColor;
    {Determines the HotTrack color of the caption}
    property DownFontColor: TColor read FDownFontColor write SetDownFontColor;
    {Determines the color of the caption when the button is down.}
    property Hint;
    {Contains the text string that can appear when the user moves the mouse over the button.}
    property ShowHint;
    {Determines whether the control displays a Help Hint when the mouse pointer rests momentarily on the control. }
    property onclick; //event
    {Occurs when the user clicks the button.}
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('janBasic', [TjanColorButton]);
end;

{ TjanColorButton }

procedure TjanColorButton.Click;
begin
  FPushDown := false;
  if FGroupIndex = 0 then
    FDown := False
  else
  begin
    if FallowAllUp then
      FDown:=not FDown
    else
      FDown := true;
  end;
  invalidate;
  inherited click;
end;

constructor TjanColorButton.Create(AOwner: TComponent);
begin
  inherited;
  width := 24;
  height := 24;
  FColor := $00FF8000;
  FDownColor := clNavy;
  FDownFontColor := clWhite;
  FPushDown := false;
  FMouseOver := false;
  FHotTrack := true;
  FHotFontColor := clYellow;
  FHotColor := $00FF8000;
  FDown := false;
  FFlat := true;
  FMarginHorizontal := 8;
  FMarginVertical := 4;
  Falignment := taCenter;
  FBackBitmap := TBitmap.Create;
  FBackBitmap.OnChange := BackBitmapChanged;
end;

procedure TjanColorButton.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FPushDown := true;
  Paint;
end;

procedure TjanColorButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;
end;

procedure TjanColorButton.Paint;
var
  R, RCap: Trect;
  w, h: integer;
  TileBack: boolean;

  procedure DrawBackBitmap;
  var
    ix, iy: Integer;
    BmpWidth, BmpHeight: Integer;
    hCanvas, BmpCanvas: THandle;
    bm: Tbitmap;
  begin
    bm := FBackBitmap;
    begin
      BmpWidth := bm.Width;
      BmpHeight := bm.Height;
      BmpCanvas := bm.Canvas.Handle;
      hCanvas := THandle(canvas.handle);
      for iy := 0 to ClientHeight div BmpHeight do
        for ix := 0 to ClientWidth div BmpWidth do
          BitBlt(hCanvas, ix * BmpWidth, iy * BmpHeight,
            BmpWidth, BmpHeight, BmpCanvas,
            0, 0, SRCCOPY);
    end;
  end;

  procedure drawcaption(fontcolor: TColor);
  begin
    canvas.brush.style := bsclear;
    canvas.font.color := Fontcolor;
    if not Fwordwrap then
    begin
      case Falignment of
        taCenter:
          DrawText(canvas.handle, @Caption[1], -1, Rcap, DT_SINGLELINE or DT_VCENTER or DT_CENTER);
        taLeftJustify:
          DrawText(canvas.handle, @Caption[1], -1, Rcap, DT_SINGLELINE or DT_VCENTER or DT_LEFT);
        taRightJustify:
          DrawText(canvas.handle, @Caption[1], -1, Rcap, DT_SINGLELINE or DT_VCENTER or DT_RIGHT);
      end;
    end
    else
    begin // wordwrap
      case Falignment of
        taCenter:
          DrawText(canvas.handle, @Caption[1], -1, Rcap, DT_VCENTER or DT_CENTER or DT_WORDBREAK);
        taLeftJustify:
          DrawText(canvas.handle, @Caption[1], -1, Rcap, DT_VCENTER or DT_LEFT or DT_WORDBREAK);
        taRightJustify:
          DrawText(canvas.handle, @Caption[1], -1, Rcap, DT_VCENTER or DT_RIGHT or DT_WORDBREAK);
      end
    end;
  end;

  procedure drawbackground(AColor: TColor);
  begin
    if TileBack then DrawBackBitmap
    else
    begin
      canvas.brush.color := AColor;
      canvas.FillRect(R);
    end;
  end;

  procedure drawdownborder;
  begin
    with canvas do
    begin
      pen.style := pssolid;
      pen.color := clgray;
      MoveTo(w, 0);
      LineTo(0, 0);
      LineTo(0, h);
      pen.color := clwhite;
      LineTo(w, h);
      LineTo(w, 0);
    end;
  end;

  procedure drawupborder;
  begin
    with canvas do
    begin
      pen.style := pssolid;
      pen.color := clwhite;
      MoveTo(w, 0);
      LineTo(0, 0);
      LineTo(0, h);
      pen.color := clgray;
      LineTo(w, h);
      LineTo(w, 0);
    end;
  end;

begin
  if assigned(FBackBitmap) and (FBackBitmap.Height <> 0) and (FBackBitmap.Width <> 0) then
    TileBack := true
  else
    TileBack := false;
  canvas.font.Assign(Font);
  autofit;
  R := Rect(0, 0, width, height);
  if not FWordwrap then
    Rcap := rect(FMarginHorizontal, 0, width - FMarginHorizontal, height)
  else
    RCap := rect(FMarginHorizontal, FMarginvertical, width - FMarginHorizontal, height - FMarginVertical);
  if FPushDown or FDown then
  begin
    RCap.left := Rcap.left + 1;
    RCap.top := RCap.top + 1;
    RCap.Right := RCap.right + 1;
    RCap.Bottom := Rcap.Bottom + 1;
  end;
  w := width - 1;
  h := height - 1;
  if (csDesigning in ComponentState) then
  begin
    drawbackground(FColor);
    drawupborder;
    drawcaption(font.color);
  end
  else if FPushDown or FDown then
  begin // depressed button
    drawbackground(FDownColor);
    drawdownborder;
    drawcaption(FDownFontColor);
    UpdateExclusive;
  end
  else if (FMouseOver and FHotTrack and (not FDown)) then
  begin // raised button with highlight caption
    drawbackground(FHotColor);
    drawupborder;
    drawcaption(FHotFontcolor);
  end
  else if FMouseOver or (not FFlat) then
  begin // raised button with normal caption
    drawbackground(Fcolor);
    drawupborder;
    drawcaption(font.color);
  end
  else
  begin // flat button with normal caption
    drawbackground(FColor);
    drawcaption(font.color);
  end;
end;

procedure TjanColorButton.CMMouseLeave(var Message: TMessage);
begin
  FMouseOver := false;
  Paint;
end;

procedure TjanColorButton.SetHotTrack(const Value: boolean);
begin
  FHotTrack := Value;
end;

procedure TjanColorButton.CMMouseEnter(var Message: TMessage);
begin
  FMouseOver := true;
  Paint;
end;

procedure TjanColorButton.SetColor(const Value: TColor);
begin
  if value <> FColor then
  begin
    FColor := Value;
    invalidate;
  end;
end;

procedure TjanColorButton.SetHotColor(const Value: TColor);
begin
  if value <> FHotColor then
  begin
    FHotColor := Value;
  end;
end;

procedure TjanColorButton.SetHotFontColor(const Value: Tcolor);
begin
  FHotFontColor := Value;
end;

procedure TjanColorButton.AutoFit;
var w, h: integer;
  R: Trect;
begin
  if not FWordwrap then
  begin
    w := canvas.TextWidth(Caption) + 2 * FMarginHorizontal;
    h := canvas.TextHeight(Caption) + 2 * FMarginVertical;
    if width < w then width := w;
    if height < h then height := h;
  end
  else
  begin
    R := rect(FMarginHorizontal, FMarginVertical, width - FMarginHorizontal, height);
    DrawText(canvas.handle, @Caption[1], -1, R, DT_CALCRECT or DT_WORDBREAK);
    R.bottom := R.Bottom + FMarginVertical;
    h := R.bottom - R.top + 1;
    if height < h then height := h;
  end;
end;

procedure TjanColorButton.SetMarginHorizontal(const Value: integer);
begin
  if value <> FMarginHorizontal then
  begin
    FMarginHorizontal := Value;
    invalidate;
  end;
end;

procedure TjanColorButton.CMFontChanged(var Message: TMessage);
begin
  invalidate;
end;

procedure TjanColorButton.CMTextChanged(var Message: TMessage);
begin
  invalidate;
end;

procedure TjanColorButton.SetAlignment(const Value: Talignment);
begin
  if value <> Falignment then
  begin
    FAlignment := Value;
    invalidate;
  end;
end;

procedure TjanColorButton.SetMarginVertical(const Value: integer);
begin
  if value <> FMarginVertical then
  begin
    FMarginVertical := Value;
    invalidate;
  end;
end;

procedure TjanColorButton.SetWordWrap(const Value: boolean);
begin
  if value <> FWordWrap then
  begin
    FWordWrap := Value;
    invalidate;
  end;
end;

procedure TjanColorButton.SetBackBitmap(const Value: TBitmap);
begin
  FBackBitmap.Assign(Value);
end;

destructor TjanColorButton.Destroy;
begin
  FBackBitmap.Free;
  inherited;
end;

procedure TjanColorButton.BackBitmapChanged(Sender: TObject);
begin
  invalidate;
end;


procedure TjanColorButton.SetDown(const Value: boolean);
begin
  if value <> FDown then
  begin
    FDown := Value;
    invalidate;
  end;
  // Leo
  if Value then UpdateExclusive;
end;

procedure TjanColorButton.SetDownColor(const Value: TColor);
begin
  if value <> FDownColor then
  begin
    FDownColor := Value;
  end;
end;

procedure TjanColorButton.SetDownFontColor(const Value: TColor);
begin
  FDownFontColor := Value;
end;

procedure TjanColorButton.SetFlat(const Value: boolean);
begin
  if value <> FFlat then
  begin
    FFlat := Value;
    invalidate;
  end;
end;

// Modficado por Leo

procedure TjanColorButton.SetGroupIndex(Value: Integer);

begin
  if FGroupIndex <> Value then
  begin
    FGroupIndex := Value;
    UpdateExclusive;
  end;
end;

procedure TjanColorButton.UpdateExclusive;

var Msg: TMessage;

begin
  if (FGroupIndex <> 0) and (Parent <> nil) then
  begin
    Msg.Msg := CM_BUTTONPRESSED;
    Msg.WParam := FGroupIndex;
    Msg.LParam := Longint(Self);
    Msg.Result := 0;
    Parent.Broadcast(Msg);
  end;
end;

//-----------------------------------------------------------------------------

procedure TjanColorButton.CMButtonPressed(var Message: TMessage);

var Sender: TjanColorButton;

begin
  if Message.WParam = FGroupIndex then
  begin
    Sender := TjanColorButton(Message.LParam);
    if Sender <> Self then
    begin
      if Sender.Down and FDown then
      begin
        FDown := False;
        Invalidate;
      end;
    end;
  end;
end;

procedure TjanColorButton.SetAllowAllUp(const Value: boolean);
begin
  FAllowAllUp := Value;
end;

procedure TjanColorButton.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and Enabled and Visible and
      (Parent <> nil) and Parent.Showing then
    begin
      Click;
      Result := 1;
    end else
      inherited;
end;

end.

