unit janOutBar;


interface

uses Messages, Windows, SysUtils, CommCtrl, Classes, Controls, Forms,
  Menus, Graphics, StdCtrls, ToolWin, ImgList, ExtCtrls;

type



{ TjanOutBar }
  TjanOutButtonState=(jobsRaised,jobsFlat,jobsSunken);

  TjanOutBar = class;
  TjanOutPanel = class;
  TjanOutButtons=class;

  TjanOutButton = class(TCollectionItem)
  private
    FDisplayRect:TRect;
    FButtonRect:TRect;
    FText: string;
    FImageIndex: integer;
    FonClick: TNotifyEvent;
    procedure SetText(const Value: string);
    procedure SetImageIndex(const Value: integer);
    procedure SetonClick(const Value: TNotifyEvent);
  protected
    function GetDisplayName: string; override;
    procedure Click;
  public
    constructor Create(Collection: TCollection); override;
    procedure Assign(Source: TPersistent); override;
  published
    property Text: string read FText write SetText;
    property ImageIndex:integer read FImageIndex write SetImageIndex;
    property onClick:TNotifyEvent read FonClick write SetonClick;
  end;

  TjanOutButtons = class(TCollection)
  private
    FjanOutPanel: TjanOutPanel;
    function GetItem(Index: Integer): TjanOutButton;
    procedure SetItem(Index: Integer; Value: TjanOutButton);
  protected
    function GetOwner: TPersistent; override;
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(janOutPanel: TjanOutPanel);
    function Add: TjanOutButton;
    property Items[Index: Integer]: TjanOutButton read GetItem write SetItem; default;
  end;


  TjanOutPanel = class(TCollectionItem)
  private
    FDisplayRect:TRect;
    FText: string;
    FButtons: TjanOutButtons;
    FonClick: TNotifyEvent;
    FSmallImages: boolean;
    procedure SetText(const Value: string);
    procedure SetButtons(const Value: TjanOutButtons);
    procedure SetonClick(const Value: TNotifyEvent);
    procedure SetSmallImages(const Value: boolean);
  protected
    function GetDisplayName: string; override;
    procedure Click;
  public
    constructor Create(Collection: TCollection); override;
    destructor  Destroy;override;
    procedure Assign(Source: TPersistent); override;
  published
    property Text: string read FText write SetText;
    property Buttons: TjanOutButtons read FButtons write SetButtons;
    property onClick:TNotifyEvent read FonClick write SetonClick;
    property SmallImages:boolean read FSmallImages write SetSmallImages;
  end;

  TjanOutPanels = class(TCollection)
  private
    FjanOutBar: TjanOutBar;
    function GetItem(Index: Integer): TjanOutPanel;
    procedure SetItem(Index: Integer; Value: TjanOutPanel);
  protected
    function GetOwner: TPersistent; override;
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(janOutBar: TjanOutBar);
    function Add: TjanOutPanel;
    property Items[Index: Integer]: TjanOutPanel read GetItem write SetItem; default;
  end;

  TDrawPanelEvent = procedure(janOutBar: TjanOutBar; Panel: TjanOutPanel;
    const Rect: TRect) of object;

  TjanOutBar = class(TCustomControl)
  private
    FPanels: TjanOutPanels;
    FOutRect:TRect;  // rectangle for active panel buttons
    FScrollTimer:TTimer; // timer for scroll repeat
    FScrollTop:integer; // first visible button
    FScrollBottom:integer; // last visible button
    FUpR:TRect;  // rect for up scroll button
    FUpActive:boolean; //upscroller visible
    FUpIsDown:boolean; // upscroller depressed
    FDownR:TRect; // rect for down scroll button
    FDownActive:boolean; // downscroller visible
    FDownIsDown:boolean; // downscroller depressed
    FPanelIndex: integer; // current panel
    FTabHeight: integer; // height of the clickable tabs
    FOutHeight: integer; // current height of the buttons area
    FDownTab:integer;   // current mousedown Tab
    FDownButton:integer; // current mousedown Button
    FMoveButton:integer; // current mousemove Button
    FMoveTab:integer;  // current mousemove Tab
    FImages: TImagelist; // large images, caption displayed below image
    FSmallImages: TImageList; // small images, caption displayed right of image
    procedure SetPanels(Value: TjanOutPanels);
    procedure UpdatePanel(Index: Integer; State:TjanOutButtonState);
    procedure UpdatePanels(UpdateRects:boolean);
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    function GetPanelAtPos(APoint: TPoint): integer;
    procedure SetPanelIndex(const Value: integer);
    procedure SetTabHeight(const Value: integer);
    procedure SetImages(const Value: TImagelist);
    procedure UpdateOutPanel(First:boolean;index: integer);
    procedure UpdateButton(aPanel, aButton: integer;state:TjanOutButtonState);
    function GetButtonAtPos(pt: TPoint):integer;
    procedure SetSmallImages(const Value: TImageList);
    function GetButtonArea(pt: TPoint): integer;
    procedure DrawDownRaised;
    procedure DrawDownSunken;
    procedure DrawUpRaised;
    procedure DrawUpSunken;
    procedure DrawRaised(R: TRect);
    procedure DrawUpArrow(R:TRect);
    procedure DrawDownArrow(R: TRect);
    procedure DrawSunken(R: TRect);
    procedure TimedScroll(sender:TObject);
  protected
    procedure CMDesignHitTest( var Msg: TCMDesignHitTest ); message CM_DESIGNHITTEST;
    procedure ClickAtPos( APoint: TPoint );
    procedure CreateWnd; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Notification(AComponent:TComponent; Operation:TOperation);override;
//    property Canvas: TCanvas read FCanvas;
  published
    property Align default alLeft;
    property Enabled;
    property Images:TImagelist read FImages write SetImages;
    property SmallImages:TImageList read FSmallImages write SetSmallImages;
    property Panels: TjanOutPanels read FPanels write SetPanels;
    property PanelIndex:integer read FPanelIndex write SetPanelIndex;
    property TabHeight:integer read FTabHeight write SetTabHeight;
    property PopupMenu;
    property ShowHint;
    property Hint;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
  end;

function InitCommonControl(CC: Integer): Boolean;


procedure Register;

implementation

uses Consts, StdActns;

procedure Register;
begin
  RegisterComponents('janBasic', [TjanOutBar]);
end;


function InitCommonControl(CC: Integer): Boolean;
var
  ICC: TInitCommonControlsEx;
begin
  ICC.dwSize := SizeOf(TInitCommonControlsEx);
  ICC.dwICC := CC;
  Result := InitCommonControlsEx(ICC);
  if not Result then InitCommonControls;
end;

{ TjanOutPanel }

constructor TjanOutPanel.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FButtons:=TjanOutButtons.create(self);  
end;

procedure TjanOutPanel.Assign(Source: TPersistent);
begin
  if Source is TjanOutPanel then
  begin
    Text := TjanOutPanel(Source).Text;
  end
  else inherited Assign(Source);
end;


function TjanOutPanel.GetDisplayName: string;
begin
  Result := Text;
  if Result = '' then Result := inherited GetDisplayName;
end;


procedure TjanOutPanel.SetText(const Value: string);
begin
  if FText <> Value then
  begin
    FText := Value;
    Changed(False);
  end;
end;




procedure TjanOutPanel.SetButtons(const Value: TjanOutButtons);
begin
  FButtons.assign(Value);
end;

destructor TjanOutPanel.Destroy;
begin
  FButtons.Free;
  inherited;

end;

procedure TjanOutPanel.SetonClick(const Value: TNotifyEvent);
begin
  FonClick := Value;
end;

procedure TjanOutPanel.Click;
begin
  if assigned(onClick) then
    onclick(self);
end;

procedure TjanOutPanel.SetSmallImages(const Value: boolean);
begin
  if FsmallImages<>value then
  begin
    FSmallImages := Value;
    changed(true);
  end;
end;

{ TjanOutPanels }

constructor TjanOutPanels.Create(janOutBar: TjanOutBar);
begin
  inherited Create(TjanOutPanel);
  FjanOutBar := janOutBar;
end;

function TjanOutPanels.Add: TjanOutPanel;
begin
  Result := TjanOutPanel(inherited Add);
end;

function TjanOutPanels.GetItem(Index: Integer): TjanOutPanel;
begin
  Result := TjanOutPanel(inherited GetItem(Index));
end;

function TjanOutPanels.GetOwner: TPersistent;
begin
  Result := FjanOutBar;
end;

procedure TjanOutPanels.SetItem(Index: Integer; Value: TjanOutPanel);
begin
  inherited SetItem(Index, Value);
end;

procedure TjanOutPanels.Update(Item: TCollectionItem);
begin
  if Item <> nil then
    FjanOutBar.UpdatePanel(Item.Index, jobsflat) else
    FjanOutBar.UpdatePanels(True);
end;


{ TjanOutBar }

constructor TjanOutBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
//  ControlStyle := [csCaptureMouse, csClickEvents, csDoubleClicks, csOpaque];
  Color := clBtnFace;
  Width := 150;
  Align := alLeft;
  FTabHeight:=23;
  FPanels := TjanOutPanels.Create(Self);
  ParentFont := False;
  FMoveButton:=-1;
  FMoveTab:=-1;
end;

destructor TjanOutBar.Destroy;
begin
  FPanels.Free;
  inherited Destroy;
end;


procedure TjanOutBar.CreateWnd;
begin
  inherited CreateWnd;
  UpdatePanels(True);
end;



procedure TjanOutBar.SetPanels(Value: TjanOutPanels);
begin
  FPanels.Assign(Value);
end;

procedure TjanOutBar.UpdatePanel(Index: Integer; State:TjanOutButtonState);
var
  Flags: Integer;
  S: string;
  R,R2: TRect;
begin
  if HandleAllocated then
    with Panels[Index] do
    begin
      s:=Text;
      R:=FDisplayRect;
      canvas.brush.color:=clSilver;
      canvas.FillRect (R);
      canvas.brush.style:=bsclear;
      case state of
      jobssunken :  Frame3D(canvas,R,clbtnshadow,clbtnHighlight,2);
      jobsflat   :  Frame3D(canvas,R,cl3DLight,clbtnshadow,1);
      jobsraised :
        begin
          Frame3D(canvas,R,clbtnHighlight,clblack,1);
        end;
      end;
      canvas.font.color:=clblack;
      drawtext(canvas.handle,@s[1],-1,R,DT_SINGLELINE or DT_CENTER or DT_VCENTER);
    end;
end;



procedure TjanOutBar.UpdateButton(aPanel,aButton:integer;state:TjanOutButtonState);
var R,btnR:Trect;
    btn:TjanOutButton;
    img,h,w,dx,dy:integer;
    s:string;
    small:boolean;
begin
  if Panels.Count=0 then exit;
  if Panels[aPanel].buttons.Count=0 then exit;
  btn:=Panels[aPanel].Buttons[aButton];
  R:=btn.FDisplayRect;
  btnR:=btn.FButtonRect;
  small:=Panels[aPanel].SmallImages;
  img:=btn.ImageIndex;
//  TjanOutButtonState=(jobsRaised,jobsFlat,jobsSunken);
  case state of
    jobssunken: frame3D(canvas,btnR,clblack,clsilver,1);
    jobsraised: frame3D(canvas,btnR,clsilver,clBlack,1);
    jobsflat:   frame3D(canvas,btnR,clgray,clgray,1);
  end;
  canvas.font.color:=clwhite;
  if small then
  begin
    w:=16;
    h:=16;
    if assigned(FSmallImages) then
    begin
      w:=FsmallImages.Width;
      h:=FSmallImages.height;
    end;
  end
  else begin
    w:=24;
    h:=24;
    if assigned(FImages) then
    begin
      w:=FImages.Width;
      h:=FImages.height;
    end;
  end;
  dx:=(btnR.Right-BtnR.Left-w) div 2;
  dy:=(btnR.Bottom-btnR.top-h) div 2;
  if small then
    FSmallImages.Draw(canvas,btnR.left+dx,btnR.top+dy,img)
  else
  begin
    Fimages.Draw(canvas,btnR.left+dx,btnR.top+dy,img);
  end;
  s:=btn.Text;
  if small then
  begin
    R.left:=btnR.right+4;
    drawtext(canvas.handle,@s[1],-1,R,DT_SINGLELINE or DT_LEFT);
  end
  else
  begin
    R.top:=btnR.Bottom+2;
   drawtext(canvas.handle,@s[1],-1,R,DT_SINGLELINE or DT_CENTER or DT_TOP);
  end;
end;

// drawing of scroll buttons
procedure TjanOutBar.DrawRaised(R:TRect);
begin
  with canvas do
  begin
    brush.color:=clsilver;
    Fillrect(R);
    pen.color:=cl3DLight;
    moveto(R.Right-1,R.top);
    lineto(R.left,R.top);
    lineto(R.left,R.bottom-1);
    pen.color:=clBlack;
    moveto(R.left,R.bottom);
    lineto(R.right,R.bottom);
    lineto(R.right,R.top);
    pen.color:=clwhite;
    moveto(R.right-2,R.top+1);
    lineto(R.left+1,R.top+1);
    lineto(R.left+1,R.bottom-2);
    pen.color:=clgray;
    moveto(R.left+1,R.bottom-1);
    lineto(R.right-1,R.bottom-1);
    lineto(R.right-1,R.top+1);
    brush.style:=bsclear;
  end;
end;

procedure TjanOutBar.DrawSunken(R:TRect);
begin
  with canvas do
  begin
    brush.color:=clsilver;
    Fillrect(R);
    pen.color:=clblack;
    moveto(R.Right-1,R.top);
    lineto(R.left,R.top);
    lineto(R.left,R.bottom-1);
    pen.color:=cl3DLight;
    moveto(R.left,R.bottom);
    lineto(R.right,R.bottom);
    lineto(R.right,R.top);
    pen.color:=clgray;
    moveto(R.right-2,R.top+1);
    lineto(R.left+1,R.top+1);
    lineto(R.left+1,R.bottom-2);
    pen.color:=clsilver;
    moveto(R.left+1,R.bottom-1);
    lineto(R.right-1,R.bottom-1);
    lineto(R.right-1,R.top+1);
    brush.style:=bsclear;    
  end;
end;


procedure TjanOutBar.DrawUpArrow(R:TRect);
begin
  with canvas do
  begin
    pen.color:=clblack;
    moveto(R.left+5,R.top+11);
    Lineto(R.left+13,R.top+11);
    moveto(R.left+6,R.top+10);
    Lineto(R.left+12,R.top+10);
    moveto(R.left+7,R.top+9);
    Lineto(R.left+11,R.top+9);
    moveto(R.left+8,R.top+8);
    Lineto(R.left+10,R.top+8);
    moveto(R.left+9,R.top+7);
    Lineto(R.left+9,R.top+7);
  end;
end;

procedure TjanOutBar.DrawDownArrow(R:TRect);
begin
  with canvas do
  begin
    pen.color:=clblack;
    moveto(R.left+5,R.top+8);
    Lineto(R.left+13,R.top+8);
    moveto(R.left+6,R.top+9);
    Lineto(R.left+12,R.top+9);
    moveto(R.left+7,R.top+10);
    Lineto(R.left+11,R.top+10);
    moveto(R.left+8,R.top+11);
    Lineto(R.left+10,R.top+11);
    moveto(R.left+9,R.top+12);
    Lineto(R.left+9,R.top+12);
  end;
end;


procedure TjanOutBar.DrawUpRaised;
begin
  DrawRaised(FUpR);
  DrawUpArrow(FUpR);
end;

procedure TjanOutBar.DrawUpSunken;
begin
  DrawSunken(FUpR);
  DrawUpArrow(FUpR);
end;

procedure TjanOutBar.DrawDownRaised;
begin
  DrawRaised(FDownR);
  DrawDownArrow(FDownR);
end;

procedure TjanOutBar.DrawDownSunken;
begin
  DrawSunken(FDownR);
  DrawDownArrow(FDownR);
end;


procedure TjanOutBar.UpdateOutPanel(First:boolean;index:integer);
var i,count,ButtonPos,w,h,dx:integer;
    R,btnR,displR:TRect;
    small:boolean;
    ZoneHeight:integer;
    ZoneR:TRect;
    Timed:boolean;
begin
  R:=FOutRect;
  canvas.brush.color:=clgray;
  canvas.FillRect (FOutRect);
  FUpR:= rect(R.right-25,R.top+5,R.right-6,R.top+24);
  FUpActive:=false;
  FDownR:=rect(R.right-25,R.bottom-25,R.right-6,R.bottom-6);
  FDownActive:=false;
  small:= Panels[index].SmallImages;
  count:=panels[index].Buttons.count;
  if count=0 then exit;
  ButtonPos:=FOutRect.top+2;
  if small then
  begin
    if assigned(FSmallImages) then
    begin
      h:=FSmallImages.height+2;
      w:=FSmallImages.width+2;
      ZoneHeight:=h+7;
      ZoneR:=rect(R.left+2,ButtonPos,R.Right-2,ButtonPos+ZoneHeight-1);
      btnR:=rect(ZoneR.left+2,ZoneR.Top+2,ZoneR.Left+4+w,ZoneR.top+4+h);
    end
    else
     exit; // only display buttons when images assigned
  end
  else begin
    if assigned(FImages) then
    begin
      h:=FImages.Height+6;
      w:=FImages.Width+6;
      ZoneHeight:=h+22;
      ZoneR:=rect(R.left+2,ButtonPos,R.Right-2,ButtonPos+ZoneHeight-1);
      dx:=(R.right-R.left-4-w) div 2;
      btnR:=rect(ZoneR.left+dx,ZoneR.Top+4,ZoneR.Left+dx+4+w,ZoneR.top+6+h);
    end
    else
     exit; // only display buttons when images assigned
  end;
  if First then FScrollTop:=0;
  FScrollBottom:=FScrollTop+((FOutRect.Bottom-FOutRect.top+1) div ZoneHeight)-1;
  if FScrollBottom>(count-1) then
   FScrollBottom:=count-1;
  // set button rects
  if Panels[index].Buttons.count>FScrollBottom then
    for I := FScrollTop to FScrollBottom do
    begin
      Panels[index].Buttons[i].FDisplayRect :=ZoneR;
      Panels[index].Buttons[i].FButtonRect:=btnR;
    // next zone
      inc(ZoneR.top,ZoneHeight);
      inc(zoneR.bottom,Zoneheight);
      inc(btnR.top,ZoneHeight);
      inc(btnR.bottom,Zoneheight);
    end;
  // display buttons
  if Panels[index].Buttons.count>FScrollBottom then
    for i:=FScrollTop to FScrollBottom do
      UpdateButton(index,i,jobsFlat);
    // check display of scroll buttons
//  timed:= assigned(FScrollTimer) and (FScrollTimer.enabled);
  if FScrollTop>0 then
  begin
    FUpActive:=true;
    if FUpIsDown then
      DrawUpSunken
    else
      DrawUpRaised;
  end
  else
    FUpActive:=false;
  if FScrollBottom<(count-1)then
  begin
    FDownActive:=true;
    if FDownIsDown then
      DrawDownSunken
    else
      DrawDownRaised;
  end
  else
   FDownActive:=false;
end;


procedure TjanOutBar.UpdatePanels(UpdateRects:boolean);
const
  MaxPanelCount = 128;
var
  I, Count, PanelPos,ot: Integer;
  R:TRect;
  PanelEdges: array[0..MaxPanelCount - 1] of Integer;
begin
  if HandleAllocated then
  begin
    R:=ClientRect;
    canvas.brush.color:=clgray;
    canvas.fillrect(R);
    Frame3D(canvas,R,clblack,clbtnhighlight,1);
    Count := Panels.Count;
    if UpdateRects then
    begin
      if Count > MaxPanelCount then Count := MaxPanelCount;
      if Count = 0 then
      begin
        //
      end else
      begin
        FOutHeight:=Height-2-(Count*FTabHeight);
        PanelPos := 1;
        for I := 0 to Count - 1 do
        begin
          Panels[I].FDisplayRect := rect(1,PanelPos,width-2,PanelPos+FTabHeight-1);
          Inc(PanelPos, FTabHeight);
          if i=PanelIndex then
           Inc(PanelPos, FOutHeight);
        end;
      end;
    end;
    if count>0 then
    begin
    for I := 0 to Count - 1 do
      begin
        UpdatePanel(I, jobsflat);
        if i=PanelIndex then
        begin
          ot:=Panels[i].FDisplayRect.bottom+1;
          FOutRect:=Rect(1,ot,width-2,ot+FOutHeight-1);
          UpdateOutPanel(true,i);
        end;
      end;
    end;
  end;
end;




procedure TjanOutBar.WMPaint(var Message: TWMPaint);
begin
  UpdatePanels(true);
  inherited;
end;

procedure TjanOutBar.WMSize(var Message: TWMSize);
begin
  { Eat WM_SIZE message to prevent control from doing alignment }
  if not (csLoading in ComponentState) then Resize;
  UpdatePanels(true);
end;



// -- Captures Mouse movement and Clicking during Design-Time
procedure TjanOutBar.CMDesignHitTest(var Msg: TCMDesignHitTest);
const
  ovControlClick = 1;
begin
  inherited;
  if (Msg.Keys = ovControlClick) then
    ClickAtPos( Point( Msg.Pos.x, Msg.Pos.y ) );
end;



procedure TjanOutBar.ClickAtPos(APoint: TPoint);
var
  index : integer;
begin
  index := GetPanelAtPos( APoint );
  if (index <> -1) then
    PanelIndex:=index;
end;


function TjanOutBar.GetPanelAtPos(APoint:TPoint):integer;
var i:integer;
begin
  result:=-1;
  if Panels.Count= 0 then exit;
  for i:=0 to Panels.count-1 do
    if ptInRect(Panels[i].FDisplayrect,Apoint) then
    begin
      result:=i;
      exit;
    end;
end;


procedure TjanOutBar.SetPanelIndex(const Value: integer);
begin
  FMoveButton:=-1;
  FMoveTab:=-1;
  if FPanelIndex<>value then
  begin
    FPanelIndex := Value;
    UpdatePanels(true);
  end;
end;

procedure TjanOutBar.SetTabHeight(const Value: integer);
begin
  if FTabHeight<> value then
  begin
   FTabHeight := Value;
   UpdatePanels(true);
  end;
end;

function TjanOutBar.GetButtonArea(pt:TPoint):integer;
var i:integer;
begin
  result:=-1;
  if panelIndex=-1 then exit;
  for i:=FScrollTop to FScrollBottom do
  begin
    if ptInRect(Panels[PanelIndex].buttons[i].FDisplayRect,pt) then
    begin
      result:=i;
      exit;
    end;
  end;
end;




function TjanOutBar.GetButtonAtPos(pt:TPoint):integer;
var i:integer;
begin
  result:=-1;
  if panelIndex=-1 then exit;
  if Panels[PanelIndex].Buttons.count=0 then exit;
  for i:=FScrollTop to FScrollBottom do
  begin
    if ptInRect(Panels[PanelIndex].buttons[i].FButtonRect,pt) then
    begin
      result:=i;
      exit;
    end;
  end;
end;

procedure TjanOutBar.MouseDown(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
var index:integer;
begin
  FDownTab:=-1;
  FDownButton:=-1;
  if FUpActive then // check if upscroll is pressed
  begin
    if ptInRect(FUpR,point(x,y)) then
    begin
      FUpIsDown:=true;
      DrawUpSunken;
      FScrollTimer:=TTimer.Create(self);
      FScrollTimer.Interval:=200;
      FScrollTimer.OnTimer:=TimedScroll;
      FScrollTimer.Enabled:=true;
      exit;
    end;
  end;
  if FDownActive then // check if downscroll is pressed
  begin
    if ptInRect(FDownR,point(x,y)) then
    begin
      FDownIsDown:=true;
      DrawDownSunken;
      FScrollTimer:=TTimer.Create(self);
      FScrollTimer.Interval:=200;
      FScrollTimer.OnTimer:=TimedScroll;
      FScrollTimer.Enabled:=true;
      exit;
    end;
  end;
  index:=GetPanelAtPos(point(x,y));
  if index<>-1 then
  begin
    FDownTab:=index;
    FMoveButton:=-1;
    UpdatePanel(FDownTab,jobssunken);
  end
  else
  begin
    index:=GetButtonAtPos(point(x,y));
    if index<>-1 then
    begin
      FDownButton:=index;
      UpdateButton(PanelIndex,FDownbutton,jobsSunken);
    end;
  end;
end;

procedure TjanOutBar.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
var index:integer;
begin
  if FUpIsDown then
  begin
    if Assigned(FScrollTimer) then
      FScrollTimer.free;
    FUpIsDown:=false;
//    DrawUpRaised;
    if FUpActive then
    begin
      inc(FScrollTop,-1);
      UpdateOutPanel(false,PanelIndex);
      exit;
    end;
  end;

  if FDownIsDown then
  begin
    if Assigned(FScrollTimer) then
      FScrollTimer.free;
    FDownIsDown:=false;
//    DrawDownRaised;
    if FDownActive then
    begin
      inc(FScrollTop,1);
      UpdateOutPanel(false,PanelIndex);
      exit;
    end;
  end;

  if FDownTab<>-1 then
  begin
    UpdatePanel(FDownTab,jobsflat);
    index:=FDownTab;
    FDownTab:=-1;
    PanelIndex:=index;
    Panels[PanelIndex].Click;
  end
  else
  if FDownButton<>-1 then
  begin
    UpdateButton(PanelIndex,FDownButton,jobsFlat);
    index:=FDownButton;
    FDownButton:=-1;
    Panels[PanelIndex].Buttons[index].Click;
  end;
end;


procedure TjanOutBar.SetImages(const Value: TImagelist);
begin
  FImages := Value;
end;

procedure TjanOutBar.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  if (operation=opremove) and (Acomponent=FImages) then
    SetImages(nil);
  if (operation=opremove) and (Acomponent=FSmallImages) then
    SetSmallImages(nil);

end;


procedure TjanOutBar.SetSmallImages(const Value: TImageList);
begin
  FSmallImages := Value;
end;

procedure TjanOutBar.MouseMove(Shift: TShiftState; X, Y: Integer);
var index,i:integer;
begin
  index:= GetPanelAtPos(point(x,y));
  if index<>-1 then
    cursor:=crHandPoint
  else
    cursor:=crDefault;
  if (not (ssleft in shift)) and
    (index<>FMoveTab) and (Panels.count<>0) then
    begin
      for i:=0 to Panels.Count-1 do
        if i<>index then
          UpdatePanel(i,jobsflat)
        else
          Updatepanel(i,jobsraised);
      FMoveTab:=index;
    end;
  if not (ssleft in shift) then
  if (PanelIndex<>-1) and
     (Panels[PanelIndex].buttons.count>0) then
  begin
    index:=GetButtonArea(point(x,y));
    if (index<>FMoveButton) then
    begin
      for i:=FScrollTop to FScrollBottom do
        if i<>index then
          UpdateButton(Panelindex,i,jobsFlat)
        else
          UpdateButton(panelindex,index,jobsraised);
      FMoveButton:=index;
    end;
  end;
end;

procedure TjanOutBar.TimedScroll(sender: TObject);
begin
  If FUpIsDown then
  begin
    if FUpActive then
    begin
      inc(FScrollTop,-1);
      UpdateOutPanel(false,PanelIndex);
    end;
    if not FUpActive then
      if assigned(FScrollTimer) then
        FScrollTimer.Enabled:=false;
  end
  else if FDownIsDown then
  begin
    if FDownActive then
    begin
      inc(FScrollTop,1);
      UpdateOutPanel(false,PanelIndex);
    end;
    if not FDownActive then
      if assigned(FScrollTimer) then
        FScrollTimer.Enabled:=false;
  end
  else
    if assigned(FScrollTimer) then
      FScrollTimer.enabled:=false;
end;

{ TjanOutButton }

procedure TjanOutButton.Assign(Source: TPersistent);
begin
  if Source is TjanOutButton then
  begin
    Text := TjanOutButton(Source).Text;
    ImageIndex:=TjanOutButton(source).ImageIndex;
    onClick:=TjanOutButton(source).onClick;
  end
  else inherited Assign(Source);
end;

procedure TjanOutButton.Click;
begin
  if assigned(onClick) then
    onClick(self);
end;

constructor TjanOutButton.Create(Collection: TCollection);
begin
  inherited Create(Collection);
end;

function TjanOutButton.GetDisplayName: string;
begin
  Result := Text;
  if Result = '' then Result := inherited GetDisplayName;
end;





procedure TjanOutButton.SetImageIndex(const Value: integer);
begin
  if FImageIndex<>value then
  begin
    FImageIndex := Value;
    changed(false)
  end;
end;

procedure TjanOutButton.SetonClick(const Value: TNotifyEvent);
begin
  FonClick := Value;
end;

procedure TjanOutButton.SetText(const Value: string);
begin
  if FText <> Value then
  begin
    FText := Value;
    Changed(False);
  end;

end;

{ TjanOutButtons }

function TjanOutButtons.Add: TjanOutButton;
begin
  Result := TjanOutButton(inherited Add);
end;

constructor TjanOutButtons.Create(janOutPanel: TjanOutPanel);
begin
  inherited Create(TjanOutButton);
  FjanOutPanel := janOutPanel;

end;

function TjanOutButtons.GetItem(Index: Integer): TjanOutButton;
begin
  Result := TjanOutButton(inherited GetItem(Index));
end;

function TjanOutButtons.GetOwner: TPersistent;
begin
  Result := FjanOutPanel;
end;

procedure TjanOutButtons.SetItem(Index: Integer; Value: TjanOutButton);
begin
  inherited SetItem(Index, Value);
end;

procedure TjanOutButtons.Update(Item: TCollectionItem);
begin
  if Item <> nil then
     FjanOutPanel.changed(false);
end;

end.
