unit janRadioButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TjanRadioButton = class(TCustomControl)
  private
    { Private declarations }
    FMouseOver:Boolean;
    FChecked: boolean;
    FonChange: TNotifyEvent;
    FGroupIndex: integer;
    FAlignment: TAlignMent;
    procedure SetChecked(const Value: boolean);
    procedure SetonChange(const Value: TNotifyEvent);
    procedure SetGroupIndex(const Value: integer);
    procedure Check;
    procedure SetAlignment(const Value: TAlignMent);
  protected
    { Protected declarations }
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure Click; override;
    procedure CMMouseLeave(var Message: TMessage); message CM_MouseLeave;
    procedure CMMouseEnter(var Message: TMessage); message CM_MouseEnter;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;

  public
    { Public declarations }
    constructor Create(AOwner:TComponent); override;
    procedure Paint; override;
  published
    { Published declarations }
    property Checked: boolean read FChecked write SetChecked;
    property GroupIndex: integer read FGroupIndex write SetGroupIndex;
    property Alignment:TAlignMent read FAlignment write SetAlignment;
    property Caption;
    property Font;
    property Color;
    property onChange:TNotifyEvent read FonChange write SetonChange;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('janBasic', [TjanRadioButton]);
end;

{ TjanRadioButton }

procedure TjanRadioButton.Click;
var pt:TPoint;
    R:TRect;
begin
  pt:=mouse.CursorPos;
  pt:=screentoClient(pt);
  R:=rect(0,0,height,height);
  if (pt.x>R.left) and (pt.x<R.right) and (pt.y>R.top) and (pt.y<R.Bottom) then
    Check;
end;

procedure TjanRadioButton.Check;
var wc:TWinControl;
    i,c:integer;
begin
  // uncheck any grouped TjanRadioButtons
  wc:=parent;
  for i:=0 to wc.ControlCount-1 do
    if (wc.controls[i] is TjanRadioButton) then
      if wc.controls[i]<>self then
        if TjanRadioButton(wc.controls[i]).GroupIndex=GroupIndex then
           TjanRadioButton(wc.controls[i]).checked:=false;
  FChecked:= true;
  if assigned(onchange) then
    onChange(self);
end;

procedure TjanRadioButton.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and Enabled and Visible and
      (Parent <> nil) and Parent.Showing then
    begin
      Check;
      Result := 1;
    end else
      inherited;
end;

procedure TjanRadioButton.CMFontChanged(var Message: TMessage);
begin
  invalidate;
end;

procedure TjanRadioButton.CMMouseEnter(var Message: TMessage);
begin
  FMouseOver := true;
  invalidate;
end;

procedure TjanRadioButton.CMMouseLeave(var Message: TMessage);
begin
  FMouseOver := false;
  invalidate;
end;

procedure TjanRadioButton.CMTextChanged(var Message: TMessage);
begin
  invalidate;
end;

constructor TjanRadioButton.Create(AOwner: TComponent);
begin
  inherited;
  width:=120;
  height:=22;
  Color:=clSilver;
end;


procedure TjanRadioButton.Paint;
var DC:HDC;
    R:TRect;
    s:string;
begin
  canvas.brush.color:=Color;
  canvas.FillRect (ClientRect);
  canvas.font.Assign (font);
  DC:=canvas.handle;
  R:=Rect(0,0,height,height);
  if FChecked then
    DrawFrameControl(DC,R,DFC_BUTTON,DFCS_BUTTONRADIO or DFCS_CHECKED)
  else
    DrawFrameControl(DC,R,DFC_BUTTON,DFCS_BUTTONRADIO);
  R:=Rect(height+4,0,width,height);
  s:=Caption;
  Canvas.brush.style:=bsclear;
  case FAlignment of
  taLeftJustify:
    DrawText(DC,pchar(s),-1,R,DT_LEFT or DT_VCENTER or DT_SINGLELINE);
  taCenter:
    DrawText(DC,pchar(s),-1,R,DT_CENTER or DT_VCENTER or DT_SINGLELINE);
  taRightJustify:
    DrawText(DC,pchar(s),-1,R,DT_RIGHT or DT_VCENTER or DT_SINGLELINE);
  end;


end;

procedure TjanRadioButton.SetChecked(const Value: boolean);
begin
  FChecked := Value;
  if Fchecked then Check;
  invalidate;
end;

procedure TjanRadioButton.SetGroupIndex(const Value: integer);
begin
  FGroupIndex := Value;
end;

procedure TjanRadioButton.SetonChange(const Value: TNotifyEvent);
begin
  FonChange := Value;
end;

procedure TjanRadioButton.SetAlignment(const Value: TAlignMent);
begin
  FAlignment := Value;
  invalidate;
end;

end.
